@@ -579,6 +579,7 @@ implementDistJensenShannon!(f32);
 //=======================================================================================
 
 /// Hamming distance. Implemented for u8, u16, u32, i32 and i16
+/// The distance returned is normalized by length of slices, so it is between 0. and 1.
 #[derive(Default)]
 pub struct DistHamming;
 
@@ -590,8 +591,9 @@ macro_rules! implementHammingDistance (
     impl Distance<$ty> for DistHamming  {
         fn eval(&self, va:&[$ty], vb: &[$ty]) -> f32 {
         // RUSTFLAGS = "-C opt-level=3 -C target-cpu=native"
+            assert_eq!(va.len(), vb.len());
             let norm : f32 = va.iter().zip(vb.iter()).filter(|t| t.0 != t.1).count() as f32;
-            norm
+            norm / va.len() as f32
         } // end of compute
     } // end of impl block
     )  // end of pattern matching
@@ -606,6 +608,8 @@ unsafe fn distance_hamming_i32_avx2(va:&[i32], vb: &[i32]) -> f32 {
 
 
 unsafe fn distance_hamming_i32<S: Simd> (va:&[i32], vb: &[i32]) -> f32 {
+    assert_eq!(va.len(), vb.len());
+    //
     let mut dist_simd = S::setzero_epi32();
     //
     let nb_simd = va.len() / S::VI32_WIDTH;
@@ -629,7 +633,7 @@ unsafe fn distance_hamming_i32<S: Simd> (va:&[i32], vb: &[i32]) -> f32 {
     for i in simd_length..va.len() {
         dist = dist + if va[i] != vb[i] { 1 } else {0};
     }
-    return dist as f32;
+    return dist as f32 / va.len() as f32;
 }  // end of distance_hamming_i32
 
 
@@ -641,8 +645,9 @@ impl  Distance<i32> for  DistHamming {
             unsafe { distance_hamming_i32_avx2(va,vb) }
         }
         else {
+            assert_eq!(va.len(), vb.len());
             let dist : f32 = va.iter().zip(vb.iter()).filter(|t| t.0 != t.1).count() as f32;
-            dist
+            dist / va.len() as f32
          }
     }
 } // end implementation Distance<i32>
@@ -651,6 +656,7 @@ impl  Distance<i32> for  DistHamming {
 implementHammingDistance!(u8);
 implementHammingDistance!(u16);
 implementHammingDistance!(u32);
+implementHammingDistance!(u64);
 implementHammingDistance!(i16);
 
 
@@ -664,20 +670,22 @@ pub struct DistJaccard;
 
 // contruct a 2-uple accumulator that has sum of max in first component , and sum of min in 2 component
 // stay in integer as long as possible
-
+// Note : summing u32 coming from hash values can overflow! We must go up to u64 for additions!
 macro_rules! implementJaccardDistance (
     ($ty:ty) => (
 
     impl Distance<$ty> for DistJaccard  {
         fn eval(&self, va:&[$ty], vb: &[$ty]) -> f32 {
-            let (max,min) : (u32, u32) = va.iter().zip(vb.iter()).fold((0u32,0u32), |acc, t| if t.0 > t.1 {
-                                (acc.0 + *t.0 as u32, acc.1 + *t.1 as u32) }
+            let (max,min) : (u64, u64) = va.iter().zip(vb.iter()).fold((0u64,0u64), |acc, t| if t.0 > t.1 {
+                                (acc.0 + *t.0 as u64, acc.1 + *t.1 as u64) }
                         else {
-                                (acc.0 + *t.1 as u32 , acc.1 + *t.0 as u32)
+                                (acc.0 + *t.1 as u64 , acc.1 + *t.0 as u64)
                              }
             );
             if max > 0 {
-                 1. - (min  as f32)/ (max as f32)
+                let dist = 1. - (min  as f64)/ (max as f64);
+                assert!(dist >= 0.);
+                dist as f32
             }
             else {
                 0.
@@ -1100,11 +1108,12 @@ fn test_simd_hamming_i32() {
         let between = Uniform::<i32>::from(-imax..imax);
         let va : Vec<i32> = (0..i).into_iter().map( |_| between.sample(&mut rng)).collect();
         let vb : Vec<i32> = (0..i).into_iter().map( |_| between.sample(&mut rng)).collect();
-        let simd_dist = unsafe {distance_hamming_i32::<Avx2>(&va, &vb)} as u32;
+        let simd_dist = unsafe {distance_hamming_i32::<Avx2>(&va, &vb)} as f32;
 
         let easy_dist : u32 = va.iter().zip(vb.iter()).map( |(a,b)| if a!=b { 1} else {0}).sum();
+        let easy_dist = easy_dist as f32 / va.len() as f32;
         println!("test size {:?} simd  exact = {:?} {:?}", i, simd_dist, easy_dist);
-        if easy_dist != simd_dist {
+        if (easy_dist - simd_dist).abs() > 1.0e-5 {
             println!(" jsimd = {:?} , jexact = {:?}", simd_dist, easy_dist);
             println!("va = {:?}" , va);
             println!("vb = {:?}" , vb);
